"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const date_fns_1 = require("date-fns");
const _mixins_1 = require("../../_mixins");
const timeProps = {
    time: {
        type: [Number, Date],
        default: () => Date.now()
    },
    type: {
        type: String,
        default: 'datetime'
    },
    to: {
        type: [Number, Date],
        default: () => Date.now()
    },
    unix: {
        type: Boolean,
        default: false
    },
    format: String,
    text: {
        type: Boolean,
        default: false
    }
};
exports.default = (0, vue_1.defineComponent)({
    name: 'Time',
    props: timeProps,
    setup(props) {
        const { localeRef, dateLocaleRef } = (0, _mixins_1.useLocale)('Time');
        const dateFnsOptionsRef = (0, vue_1.computed)(() => {
            return {
                locale: dateLocaleRef.value.locale
            };
        });
        const mergedTimeRef = (0, vue_1.computed)(() => {
            if (props.unix) {
                return (0, date_fns_1.fromUnixTime)(props.time);
            }
            return props.time;
        });
        const mergedToRef = (0, vue_1.computed)(() => {
            if (props.unix) {
                return (0, date_fns_1.fromUnixTime)(props.to);
            }
            return props.to;
        });
        const renderedTimeRef = (0, vue_1.computed)(() => {
            if (props.format) {
                return (0, date_fns_1.format)(mergedTimeRef.value, props.format, dateFnsOptionsRef.value);
            }
            else if (props.type === 'date') {
                return (0, date_fns_1.format)(mergedTimeRef.value, localeRef.value.dateFormat, dateFnsOptionsRef.value);
            }
            else if (props.type === 'datetime') {
                return (0, date_fns_1.format)(mergedTimeRef.value, localeRef.value.dateTimeFormat, dateFnsOptionsRef.value);
            }
            else {
                return (0, date_fns_1.formatDistance)(mergedTimeRef.value, mergedToRef.value, {
                    addSuffix: true,
                    locale: dateLocaleRef.value.locale
                });
            }
        });
        return {
            renderedTime: renderedTimeRef
        };
    },
    render() {
        return this.text
            ? (0, vue_1.createTextVNode)(this.renderedTime)
            : (0, vue_1.h)('time', [this.renderedTime]);
    }
});
